/* ******************************************************************

   class    : AppletPanel
   date     : 18.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

package AppletView;

/* ******************************************************************
   import statements
*********************************************************************/

import java.applet.*;
import java.awt.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Hashtable;

/**
 *
 * A class that provides the runtime environment for running applets.
 *
 * @author puehringer edgar
 *
 */
public class AppletPanel extends Panel implements AppletStub {

  // class variable declarations

  // instance variable declarations
  /** True if the applet has been initialized. */
  private boolean initialized = false;
  /** True if the applet has been started. */
  private boolean active = false;
  /** The handle to the applet to display. */
  protected Applet applet;
  /** The applet context. */
  protected AppletContext context;
  /** The URL where to load classes from. */
  protected URL codeBase;
  /** The URL of the hosting document. */
  protected URL documentBase;
  /** Hashtable containing the parameters from the hosting HTML document. */
  protected Hashtable params;
  /** The applet panel size. */
  protected Dimension size;

  // constructors
  /** Create a new AppletPanel object. */
  public AppletPanel(Applet applet, AppletContext context, URL codeBase) {
    this(applet,context,codeBase,null,new Hashtable());
  } // end constructor AppletPanel

  /** Create a new AppletPanel object. */
  public AppletPanel(Applet applet, AppletContext context, URL codeBase,
                       URL documentBase) {
    this(applet,context,codeBase,documentBase,new Hashtable());
  } // end constructor AppletPanel

  /** Create a new AppletPanel object. */
  public AppletPanel(Applet applet, AppletContext context, URL codeBase,
                       Hashtable params) {
    this(applet,context,codeBase,null,params);
  } // end constructor AppletPanel

  /** Create a new AppletPanel object. */
  public AppletPanel(Applet applet, AppletContext context, URL codeBase,
                       URL documentBase, Hashtable params) {
    super();
    this.applet       = applet;
    this.context      = context;
    this.codeBase     = codeBase;
    this.documentBase = documentBase;
    this.params       = params;
    setLayout(new GridLayout(1,1));
    applet.setStub(this);
    add(this.applet);
  } // end constructor AppletPanel

  // methods
  /** Initializes the embedded applet. */
  public void init() {
    if (!initialized) {
      initialized = true;
      applet.init();
      applet.invalidate();
      applet.validate();
    }
    else {
     Toolkit.getDefaultToolkit().beep();
    } // end if
  } // end init

  /** Start the embedded applet. */
  public void start() {
    if (!active) {
      if (!initialized) init();
      active = true;
      applet.start();
    }
    else {
     Toolkit.getDefaultToolkit().beep();
    } // end if
  } // end start

  /** Stops the embedded applet. */
  public void stop() {
    if (active) {
      applet.stop();
      active = false;
    }
    else {
     Toolkit.getDefaultToolkit().beep();
    } // end if
  } // end stop

  /** Destroy the embedded applet. */
  public void destroy() {
    if (initialized) {
      if (active) stop();
      applet.destroy();
    }
    else {
      Toolkit.getDefaultToolkit().beep();
    } // end if
    if (context instanceof StandaloneContext)
      ((StandaloneContext)context).stopAudioClips();
    // end if
  } // end destroy

  /** Sets the preferred size of this panel. */
  public void setPreferredSize(Dimension d) {
    size = d;
  } // end setPreferredSize

  /** Gets the preferred size of this panel. */
  public Dimension getPreferredSize() {
    if (size == null) size = super.getPreferredSize();
    return size;
  } // end getPreferredSize

  /** Gets the mininimum size of this panel. */
  public Dimension getMinimumSize() {
    return getPreferredSize();
  } // end getMinimumSize

  /** Gets the maximum size of this panel. */
  public Dimension getMaximumSize() {
    return getPreferredSize();
  } // end getMaximumSize

  // --- AppletStub --------------------------------------------------
  /**
   * Determines if the applet is active.  An applet is active just
   * before its start method is called. It becomes inactive
   * immediately after its stop method is called.
   *
   * @return True if the applet is active; false otherwise.
   */
  public boolean isActive() {
    return active;
  } // end isActive

  /**
   * Gets the document URL.
   *
   * @return The URL of the document containing the applet.
   */
  public URL getDocumentBase() {
    if (documentBase != null) return documentBase;

    String str = getCodeBase().toString();
    URL url    = null;

    if (!str.endsWith("/")) str += "/";
    str += applet.getClass().getName().replace('.','/');
    str += ".html";
    try {
      url = new URL(str);
    } catch (MalformedURLException excep) {
      excep.printStackTrace();
    }
    return url;
  } // end getDocumentBase

  /**
   * Gets the base URL.
   *
   * @return The URL of the applet.
   */
  public URL getCodeBase() {
    return codeBase;
  } // end getCodeBase

  /**
   * Returns the value of the named parameter in the HTML tag.
   *
   * @param name A parameter name.
   * @return The value of the named parameter.
   */
  public String getParameter(String name) {
    return (String)params.get(name);
  } // end getParameter

  /**
   * Gets a handler to the applet's context.
   *
   * @return The applet's context.
   */
  public AppletContext getAppletContext() {
    return context;
  } // end getAppletContext

  /**
   * Called when the applet wants to be resized.
   *
   * @param width The new requested width for the applet.
   * @param height The new requested height for the applet.
   */
  public void appletResize(int width, int height) {
    Window wnd = null;

    setPreferredSize(new Dimension(width,height));
    // pack the hosting window
    for (Container c = this.getParent(); c != null; c = c.getParent()) {
      if (c instanceof Window) {
        wnd = (Window)c;
        break;
      } // end if
    } // end for
    if (wnd != null) wnd.pack();
  } // end appletResize

} // end AppletPanel

/* ******************************************************************
   end of file
*********************************************************************/
