/* ******************************************************************

   class    : AudioClipTable
   date     : 19.08.98
   compiler : javac (jdk 1.1.6)
   modified : 9.2.99 - real audio functionallity added!

*********************************************************************/

package AppletView;

/* ******************************************************************
   import statements
*********************************************************************/

import java.applet.*;
import java.net.URL;
import java.util.Observable;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

/**
 * A class that holds a list of audio clips created from an applet.
 *
 * @author puehringer edgar
 */
public class AudioClipTable extends Observable {

  // instance variable declarations
  /** URL and state (loop or not loop) of all clips. */
  protected Hashtable clips;
  /** Vector containing all clips. */
  protected Vector clipVect;

  // constructors
  /** Create a new AudioClipTable object. */
  public AudioClipTable() {
    clips    = new Hashtable();
    clipVect = new Vector();
  } // end constructor AudioClipTable

  // methods
  /** Returns an instance of an audio clip. */
  public AudioClip getAudioClip(URL url) {
    AudioClip ac = new TrackableClip(url);

    if (ac != null) clipVect.addElement(ac);
    return ac;
  } // end getAudioClip

  /** Returns true if the audio clip specified by url is playing. */
  public boolean isPlaying(URL url) {
    return clips.containsKey(url);
  } // end isPlaying

  /** Returns true if a audio clip is playing in a loop. */
  public boolean isRepeating(URL url) {
    Object obj = clips.get(url);

    if (obj == null) return false;
    return ((Boolean)obj).booleanValue();
  } // end isRepeating

  /** Stops all audio clips. */
  public void stopAll() {
    Enumeration enum = clipVect.elements();

    while (enum.hasMoreElements()) {
      AudioClip ac = (AudioClip)enum.nextElement();

      ac.stop();
    } // end while
    clipVect.clear();
  } // end stopAll

  private void notifyObs(URL url) {
    setChanged();
    notifyObservers(url);
  } // end notifyObs

  // inner classes
  /** This class manages an audio clip, but doesn't play it really. */
  protected class TrackableClip implements AudioClip {

    // instance variable declarations
    /** The URL of the audio clip. */
    protected URL url;
    /** The real audio clip you are listening to. */
    protected AudioClip clip;

    //constructors
    public TrackableClip(URL url) {
      this.url  = url;
      this.clip = Applet.newAudioClip(url);
    } // end constructor TrackableClip

    // methods
    // - AudioClip ---------------------------------------------------
    /** Starts playing this audio clip. */
    public void play() {
      if (clip != null) clip.loop();

      Object obj      = clips.get(url);
      boolean looping = (obj == null) || ((Boolean)obj).booleanValue();

      if (!looping) return;
      clips.put(url,new Boolean(false));
      notifyObs(url);
    } // end play

    /** Starts playing this audio clip in a loop. */
    public void loop() {
      if (clip != null) clip.loop();

      Object obj      = clips.get(url);
      boolean looping = (obj != null) && ((Boolean)obj).booleanValue();

      if (looping) return;
      clips.put(url,new Boolean(true));
      notifyObs(url);
    } // end loop

    /** Stops playing this audio clip. */
    public void stop() {
      if (clip != null) clip.stop();

      if (clips.remove(url) == null) return;
      notifyObs(url);
    } // end stop

  } // end TrackableClip

} // end AudioClipTable

/* ******************************************************************
   end of file
*********************************************************************/
