/* ******************************************************************

   class    : BuildAction
   date     : 16.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.util.Observable;

import Messages.LockMessage;
import Messages.UnlockMessage;
import Messages.TerminateMessage;


/**
 * Build the current project.  All source files will be compiled.
 *
 * @author puehringer edgar
 */
class BuildAction extends CompileAction {

  // class variable declarations
  private static final String BUILD_ACTION = "build";

  // instance varaible declarations
  protected AbstractLinker linker;

  // constructors
  BuildAction(Frame frame, AbstractCompiler compiler,
              AbstractLinker linker) {
    super(BUILD_ACTION,frame,compiler);
    this.linker = linker;
    setEnabled(this.linker != null);
  } // end constructor BuildAction

  // methods
  public void actionPerformed(ActionEvent e) {
    if (!conditionalSave()) return;
    compiler.postMessage(new LockMessage());
    compiler.addObserver(this);
    compiler.build();
  } // end actionPerformed

    /** Set the linker command invoked by this action. */
  public void setLinker(AbstractLinker l) {
    linker = l;
    super.setEnabled(linker != null);
  } // end setLinker

  public void update(Observable o, Object arg) {
    if (arg instanceof TerminateMessage) {
      if (o instanceof AbstractCompiler) {
        boolean success = ((TerminateMessage)arg).success;

        compiler.deleteObserver(this);
        if (success) {
          linker.addObserver(this);
          linker.link(frame);
        }
        else {
          compiler.postMessage(new UnlockMessage());
        } // end if
      }
      else { // if o instance of AbstractLinker
        linker.deleteObserver(this);
        linker.postMessage(new UnlockMessage());
      } // end if
    } // end if
  } // end update

} // end BuildAction

/* ******************************************************************
   end of file
*********************************************************************/
