/* ******************************************************************

   class    : DefaultExecute
   date     : 21.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.util.ResourceBundle;
import java.util.Properties;
import java.util.StringTokenizer;

import AppletView.AppletViewDlg;
import CustCtrl.BaseDialog;
import CustCtrl.InsetsPanel;
import CustCtrl.SeparatorCanvas;
import Utils.ResourceUtil;
import Messages.ErrorMessage;
import SysExt.OutputRedirector;
import SysExt.InputRedirector;

import javax.swing.JOptionPane;

/**
 * A plugin for running java applications and applets.
 *
 * @author puehringer edgar
 */
public class DefaultExecute extends AbstractExecute {

  // class variable declarations
  public final static String SELECTDLG_TITLE   = "exec.selectDlg.title";
  public final static String SELECTDLG_BUTTONS = "exec.selectDlg.buttons";
  public final static String SELECTDLG_TEXT    = "exec.selectDlg.text";
  public final static String NOEXECDLG_TITLE   = "exec.noExecDlg.title";
  public final static String NOEXECDLG_BUTTONS = "exec.noExecDlg.buttons";
  public final static String NOEXECDLG_TEXT    = "exec.noExecDlg.text";
  /** Key to the current executable file resource. */
  private final static String CUR_FILE = "exec.currentFile";
  /** Key to the applet viewer dialog prefix. */
  private final static String APPLETVIEW_DLG = "exec.appletViewDlg.";

  /** Key to the selected applet viewer resource. */
  private final static String SELECTED_VIEWER = "exec.viewer";

  // Keys to virtual machine settings
  public final static String VM_COMMAND         = "exec.vm.command";
  public final static String VM_ADDPARAM        = "exec.vm.addParam";
  public final static String VM_USECLASSPATH    = "exec.vm.useClassPath";
  public final static String VM_CLASSPATHPREFIX = "exec.vm.classPathPrefix";
  public final static String VM_TRAILINGSPACE   = "exec.vm.trailingSpace";
  public final static String VM_MULTIINSTANCE   = "exec.vm.multiInstance";

  // Keys to applet viewer settings
  public final static String AV_COMMAND         = "exec.av.command";
  public final static String AV_ADDPARAM        = "exec.av.addParam";
  public final static String AV_USECLASSPATH    = "exec.av.useClassPath";
  public final static String AV_CLASSPATHPREFIX = "exec.av.classPathPrefix";
  public final static String AV_TRAILINGSPACE   = "exec.av.trailingSpace";
  public final static String AV_MULTIINSTANCE   = "exec.av.multiInstance";

  // Keys to a java enabled html browser settings
  public final static String BR_COMMAND         = "exec.br.command";
  public final static String BR_ADDPARAM        = "exec.br.addParam";
  public final static String BR_USECLASSPATH    = "exec.br.useClassPath";
  public final static String BR_CLASSPATHPREFIX = "exec.br.classPathPrefix";
  public final static String BR_TRAILINGSPACE   = "exec.br.trailingSpace";
  public final static String BR_MULTIINSTANCE   = "exec.br.multiInstance";

  // Keys to the settings dialog
  public final static String SETTDLG_TITLE   = "exec.settingsDlg.title";
  public final static String SETTDLG_BUTTONS = "exec.settingsDlg.buttons";
  public final static String SETTDLG_VLABEL  = "exec.settingsDlg.viewerLabel";
  public final static String SETTDLG_VCHOICE = "exec.settingsDlg.viewerChoice";
  public final static String SETTDLG_VMTITLE = "exec.settingsDlg.vmTitle";
  public final static String SETTDLG_AVTITLE = "exec.settingsDlg.avTitle";
  public final static String SETTDLG_BRTITLE = "exec.settingsDlg.brTitle";
  public final static String SETTDLG_PATH    = "exec.settingsDlg.path";
  public final static String SETTDLG_ADDPARA = "exec.settingsDlg.addPara";
  public final static String SETTDLG_USEPATH = "exec.settingsDlg.useCPath";
  public final static String SETTDLG_PREFIX  = "exec.settingsDlg.prefix";
  public final static String SETTDLG_SPACES  = "exec.settingsDlg.spaces";
  public final static String SETTDLG_MULTI   = "exec.settingsDlg.multi";

  // instance variable declarations
  /** The current selected executable file. */
  protected File curFile;
  /** True if it's recommendable to wait on the termination of subprocesses. */
  protected boolean waitForSubProcess;

  // constructors
  /** Create a new DefaultExecute class. */
  public DefaultExecute(ResourceBundle resources, Properties project) {
    super(resources,project);
  } // end constructor DefaultExecute

  // class methods
  public static void main(String[] args) {
    Frame frame = new Frame();
    DefaultExecute exec;
    SettingsDlg dlg;
    ResourceBundle res;
    Properties proj = new Properties();

    if (args.length < 1) {
      System.out.println("Resource file must be specified.");
      System.exit(1);
    } // end if
    res  = ResourceBundle.getBundle(args[0],java.util.Locale.getDefault());
    exec = new DefaultExecute(res,proj);
    dlg  = exec.new SettingsDlg(frame);
    dlg.show();
    System.out.println("isCanceled returned " + dlg.isCanceled());
    System.exit(0);
  } // end main

  // methods
  /**
   * Execute a file.
   *
   * @param frame A frame that can be used as parent for dialogs.
   * @see getProjectFiles
   * @see getParameters
   * @see notifyTermination
   */
  public void defaultExecute(Frame frame) {
    internalExecute(frame,false);
  } // end defaultExecute

  /**
   * Execute a file selected by the user.
   *
   * @param frame A frame that can be used as parent for dialogs.
   * @see getProjectFiles
   * @see getParameters
   * @see notifyTermination
   */
  public void selectedExecute(Frame frame) {
    internalExecute(frame,true);
  } // end selectedExecute

  private final void internalExecute(Frame frame, boolean select) {
    String str   = project.getProperty(CUR_FILE);
    File files[] = getProjectFiles();
    int i;
    SelectDlg dlg;
    String[] cmdLine;
    boolean isApplication = false;

    if (str == null) curFile = null;
      else curFile = new File(ResourceUtil.urlToFile(str));
    i = curFile == null ? files.length : 0;
    while (i < files.length) {
      if (files[i].getAbsolutePath().equals(curFile.getAbsolutePath())) break;
      i++;
    } // end while
    if (i >= files.length) curFile = null;
    if (files.length == 0) {
      JOptionPane.showMessageDialog(frame,
        ResourceUtil.getResStringOrKey(resources,NOEXECDLG_TEXT),
        ResourceUtil.getResStringOrKey(resources,NOEXECDLG_TITLE),
        JOptionPane.INFORMATION_MESSAGE);
      notifyTermination(0,null);
      return;
    } // end if
    // select a file
    if ((curFile == null) || select) {
      if (files.length == 1) {
        curFile = files[0];
      }
      else {
        dlg = new SelectDlg(frame,files);
        dlg.show();
        if (dlg.isCanceled()) {
          notifyTermination(0,null);
          return;
        } // end if
        curFile = files[dlg.list.getSelectedIndex()];
      } // end if
      // save current executable file to project properties
      str = ResourceUtil.fileToURL(curFile.getAbsolutePath());
      if (!str.equals(project.getProperty(CUR_FILE))) {
        project.put(CUR_FILE,str);
        setProjectChanged();
      } // end if
    } // end if
    // execute the file
    cmdLine = buildCmdLine(curFile);
    if (cmdLine == null) executeApplet(frame);
    else executeExternalTool(cmdLine,curFile);
  } // end internalExecute

  /** Execute an applet. */
  protected void executeApplet(Frame frame) {
    AppletViewDlg dlg = new AppletViewDlg(frame,resources,
                          APPLETVIEW_DLG,curFile);
    SwitchableSecurityManager sm;

    sm = (SwitchableSecurityManager)System.getSecurityManager();
    sm.setEnabled(true);
    dlg.show();
    sm.setEnabled(false);
    notifyTermination(dlg.getExitValue(),curFile);
  } // end executeApplet

  /** Execute an application. */
  protected void executeExternalTool(String[] cmdArray, File file) {
    Process proc;
    int exitValue = 0;

    try {
      proc = Runtime.getRuntime().exec(cmdArray);
      new OutputRedirector(proc,false,true);
      new OutputRedirector(proc,true,true);
      new InputRedirector(proc);
    } catch (IOException excep) {
      postMessage(new ErrorMessage(ErrorMessage.ERROR,
                    "Can't execute \"" + cmdArray[0] +
                    "\": " + excep));
      proc      = null;
      exitValue = 100;
    }
    if (proc != null) try {
      if (waitForSubProcess) exitValue = proc.waitFor();
    } catch (InterruptedException excep) {
      excep.printStackTrace();
      proc.destroy();
      exitValue = 101;
    }
    if (waitForSubProcess) notifyTermination(exitValue,file);
      else notifyTermination(0,null);
  } // end executeExternalTool

  /** Displays a settings dialog. */
  public void settings(Frame parent) {
    SettingsDlg dlg = new SettingsDlg(parent);

    dlg.show();
    if (!dlg.isCanceled()) {
      dlg.storeResults();
      setProjectChanged();
    } // end if
  } // end settings

  /**
   * Return an array containing the command line.  Return null
   * if the internal applet viewer should be executed.
   */
  protected String[] buildCmdLine(File file) {
    String extensions[]   = getExtensions();
    boolean isApplication = false;
    String str;
    int selection;
    StringTokenizer tok;
    int i,j;
    String path;
    String[] result;
    String[] params;

    // assume that the first half of the extensions referes
    // applications, the second half applets
    for (i=0; i<(extensions.length / 2); i++) {
      if (file.getAbsolutePath().endsWith(extensions[i])) {
        isApplication = true;
        break;
      } // end if
    } // end for
    if (isApplication) {
      selection = 3;
    }
    else {
      str = getSetting(SELECTED_VIEWER);
      if (str.equals("")) str = "0";
      selection = Integer.parseInt(str);
    } // end if
    if (selection == 0) return null; // if internal applet viewer
    switch (selection) {
      case 1:                         // applet viewer
        str  = getSetting(AV_COMMAND) + " ";
        str += (getSetting(AV_ADDPARAM) + " ");
        if ((new Boolean(getSetting(AV_USECLASSPATH))).booleanValue()) {
          str += getSetting(AV_CLASSPATHPREFIX);
          if ((new Boolean(getSetting(AV_TRAILINGSPACE))).booleanValue()) {
            str += " ";
          } // end if
          str += getProjectDirectory();
          str += File.pathSeparator;
          str += System.getProperty("java.class.path");
        } // end if
        path = ResourceUtil.fileToURL(file.getAbsolutePath());
        waitForSubProcess = !(new Boolean(getSetting(AV_MULTIINSTANCE))).booleanValue();
      break;
      case 2:                         // html browser
        str  = getSetting(BR_COMMAND) + " ";
        str += (getSetting(BR_ADDPARAM) + " ");
        if ((new Boolean(getSetting(BR_USECLASSPATH))).booleanValue()) {
          str += getSetting(BR_CLASSPATHPREFIX);
          if ((new Boolean(getSetting(BR_TRAILINGSPACE))).booleanValue()) {
            str += " ";
          } // end if
          str += getProjectDirectory();
          str += File.pathSeparator;
          str += System.getProperty("java.class.path");
        } // end if
        path = ResourceUtil.fileToURL(file.getAbsolutePath());
        waitForSubProcess = !(new Boolean(getSetting(BR_MULTIINSTANCE))).booleanValue();
      break;
      default:                        // application
        str  = getSetting(VM_COMMAND) + " ";
        str += (getSetting(VM_ADDPARAM) + " ");
        if ((new Boolean(getSetting(VM_USECLASSPATH))).booleanValue()) {
          str += getSetting(VM_CLASSPATHPREFIX);
          if ((new Boolean(getSetting(VM_TRAILINGSPACE))).booleanValue()) {
            str += " ";
          } // end if
          str += getProjectDirectory();
          str += File.pathSeparator;
          str += System.getProperty("java.class.path");
        } // end if
        path = resolveName(file.getAbsolutePath());
        waitForSubProcess = !(new Boolean(getSetting(VM_MULTIINSTANCE))).booleanValue();
    } // end switch
    tok    = new StringTokenizer(str," \n\r\t",false);
    i      = tok.countTokens() + 1;
    if (selection == 3) {              // if it is an application
      params = getParameters();
      result = new String[i + params.length];
      j      = 0;
      for (; i<result.length; i++) result[i] = params[j++];
    }
    else {
      result = new String[i];
    } // end if
    i      = 0;
    while (tok.hasMoreTokens()) result[i++] = tok.nextToken();
    result[i] = path;
    if (Utils.Assert.ENABLED) {
      str = "";
      for (i=0; i<result.length; i++) str += (result[i] + " ");
      System.out.println("cmdLine: " + str);
    } // end if
    return result;
  } // end buildCmdLine

    /** Returns a class name for a file name. */
    private String resolveName(String name) {
      String path = getProjectDirectory();

      if ((name == null) || !name.endsWith(".class")) return null;
      // clip .class ending
      name = name.substring(0,name.length() - 6);
      if (name.startsWith(path)) {
        name = name.substring(path.length());
        if (name.startsWith(File.separator)) name = name.substring(1);
        return name.replace(File.separatorChar,'.');
      } // end if
      return null;
    } // end resolveName

  private final String getSetting(String key) {
    String str = project.getProperty(key);

    if (str != null) return str;
    str = ResourceUtil.getResourceString(resources,key);
    if (str != null) return str;
    return "";
  } // end getSetting

  // inner classes
  /** A dialog that allows you to select an executable file. */
  class SelectDlg extends BaseDialog implements ActionListener {

    // instance variable declarations
    private Frame parent;
    protected List list;

    // constructors
    /** Create a new SelectDlg object. */
    public SelectDlg(Frame parent, File files[]) {
      super(parent,true);
      this.parent = parent;

      Panel panel = getContentPanel();
      int pos     = 0;
      String prjDir = getProjectDirectory();

      setTitle(ResourceUtil.getResStringOrKey(resources,SELECTDLG_TITLE));
      createButtons(ResourceUtil.getResStringOrKey(resources,SELECTDLG_BUTTONS));

      GridBagLayout gridbag          = new GridBagLayout();
      GridBagConstraints constraints = new GridBagConstraints();
      panel.setLayout(gridbag);
      constraints.insets     = new Insets(10,6,0,6);
      constraints.anchor     = GridBagConstraints.WEST;
      constraints.weightx    = 1.0;
      constraints.weighty    = 1.0;
      constraints.gridwidth  = GridBagConstraints.REMAINDER;
      constraints.gridheight = GridBagConstraints.RELATIVE;

      // this tmp panel is used to let the layout look more pretty
      Panel tmp = new Panel(new GridLayout(1,0));

      tmp.add(new Label(ResourceUtil.getResStringOrKey(resources,
                          SELECTDLG_TEXT)));
      tmp.add(new Label());
      gridbag.setConstraints(tmp,constraints);
      panel.add(tmp);
      constraints.insets     = new Insets(10,6,10,6);
      constraints.fill       = GridBagConstraints.BOTH;
      constraints.gridheight = GridBagConstraints.REMAINDER;
      list = new List(Math.min(10,Math.max(files.length,5)),false);
      for (int i=0; i<files.length; i++) {
        String str = files[i].getAbsolutePath();

        if (str.startsWith(prjDir)) {
          str = str.substring(prjDir.length());
          if (str.startsWith(File.separator))
            str = str.substring(1);
          // end if
        } // end if
        list.add(str);
        if (files[i].equals(curFile)) pos = i;
      } // end for
      list.select(pos);
      list.addActionListener(this);
      gridbag.setConstraints(list,constraints);
      panel.add(list);
      pack();
      center(parent);
      setResizable(false);
    } // end constructor SelectDlg

    // --- ActionListener ---------------------------------------------
    /** Invoked when an action occurs. */
    public void actionPerformed(ActionEvent e) {
      setVisible(false);
      dispose();
    } // end actionPerformed

  } // end SelectDlg

  /** A dialog that changes the settings of this class. */
  class SettingsDlg extends BaseDialog implements ActionListener {

    // instance variable declarations
    protected Choice choice;
    protected TextField vmPath;
    protected TextField vmParam;
    protected Checkbox vmUseCP;
    protected TextField vmPrefix;
    protected Checkbox vmSpaces;
    protected Checkbox vmMulti;
    protected TextField avPath;
    protected TextField avParam;
    protected Checkbox avUseCP;
    protected TextField avPrefix;
    protected Checkbox avSpaces;
    protected Checkbox avMulti;
    protected TextField brPath;
    protected TextField brParam;
    protected Checkbox brUseCP;
    protected TextField brPrefix;
    protected Checkbox brSpaces;
    protected Checkbox brMulti;
    private Frame parent;

    // constructors
    /** Create a new SettingsDlg object. */
    public SettingsDlg(Frame parent) {
      super(parent,true);
      this.parent = parent;

      Panel panel = getContentPanel();
      Label label;
      SeparatorCanvas separator;
      String[] options;
      String str;
      int i;

      setTitle(ResourceUtil.getResStringOrKey(resources,SETTDLG_TITLE));
      createButtons(ResourceUtil.getResStringOrKey(resources,SETTDLG_BUTTONS));

      GridBagLayout gridbag          = new GridBagLayout();
      GridBagConstraints constraints = new GridBagConstraints();
      panel.setLayout(gridbag);
      constraints.insets     = new Insets(10,6,10,6);
      constraints.anchor     = GridBagConstraints.WEST;
      constraints.weightx    = 1.0;
      constraints.weighty    = 1.0;
      constraints.gridwidth  = 1;
      constraints.gridheight = 1;
      label = new Label(getSetting(SETTDLG_VLABEL));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.insets     = new Insets(10,0,10,6);
      constraints.gridwidth  = GridBagConstraints.REMAINDER;
      choice  = new Choice();
      options = ResourceUtil.splitToWords(getSetting(SETTDLG_VCHOICE));
      for (i=0; i<options.length; i++) choice.add(options[i]);
      str = getSetting(SELECTED_VIEWER);
      if (str.equals("")) str = "0";
      i = Integer.parseInt(str);
      if (i < choice.getItemCount()) choice.select(i);
      gridbag.setConstraints(choice,constraints);
      panel.add(choice);
      constraints.insets     = new Insets(0,0,10,0);
      constraints.fill       = GridBagConstraints.HORIZONTAL;
      separator = new SeparatorCanvas();
      gridbag.setConstraints(separator,constraints);
      panel.add(separator);
      constraints.insets     = new Insets(0,6,10,6);
      constraints.fill       = GridBagConstraints.NONE;
      label = new Label(getSetting(SETTDLG_VMTITLE));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.insets     = new Insets(0,12,10,6);
      constraints.gridwidth  = 1;
      label = new Label(getSetting(SETTDLG_PATH));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.insets     = new Insets(0,0,10,6);
      vmPath = new TextField(getSetting(VM_COMMAND),15);
      gridbag.setConstraints(vmPath,constraints);
      panel.add(vmPath);
      constraints.gridwidth  = GridBagConstraints.RELATIVE;
      label = new Label(getSetting(SETTDLG_ADDPARA));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.gridwidth  = GridBagConstraints.REMAINDER;
      vmParam = new TextField(getSetting(VM_ADDPARAM),15);
      gridbag.setConstraints(vmParam,constraints);
      panel.add(vmParam);
      constraints.insets     = new Insets(0,12,10,6);
      constraints.gridwidth  = 1;
      label = new Label(getSetting(SETTDLG_PREFIX));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.insets     = new Insets(0,0,10,6);
      vmPrefix = new TextField(getSetting(VM_CLASSPATHPREFIX),15);
      gridbag.setConstraints(vmPrefix,constraints);
      panel.add(vmPrefix);
      constraints.gridwidth  = GridBagConstraints.REMAINDER;
      vmSpaces = new Checkbox(getSetting(SETTDLG_SPACES),
                   (new Boolean(getSetting(VM_TRAILINGSPACE))).booleanValue());
      gridbag.setConstraints(vmSpaces,constraints);
      panel.add(vmSpaces);
      constraints.insets     = new Insets(0,12,10,6);
      constraints.gridwidth  = 1;
      vmUseCP = new Checkbox(getSetting(SETTDLG_USEPATH),
                  (new Boolean(getSetting(VM_USECLASSPATH))).booleanValue());
      gridbag.setConstraints(vmUseCP,constraints);
      panel.add(vmUseCP);
      // insert a dummy label to get some extra spacing
      constraints.insets     = new Insets(0,0,10,6);
      label = new Label("");
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.gridwidth  = GridBagConstraints.REMAINDER;
      vmMulti = new Checkbox(getSetting(SETTDLG_MULTI),
                  (new Boolean(getSetting(VM_MULTIINSTANCE))).booleanValue());
      gridbag.setConstraints(vmMulti,constraints);
      panel.add(vmMulti);

      constraints.insets     = new Insets(0,0,10,0);
      constraints.fill       = GridBagConstraints.HORIZONTAL;
      separator = new SeparatorCanvas();
      gridbag.setConstraints(separator,constraints);
      panel.add(separator);
      // appletviewer settings
      constraints.insets     = new Insets(0,6,10,6);
      constraints.fill       = GridBagConstraints.NONE;
      label = new Label(getSetting(SETTDLG_AVTITLE));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.insets     = new Insets(0,12,10,6);
      constraints.gridwidth  = 1;
      label = new Label(getSetting(SETTDLG_PATH));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.insets     = new Insets(0,0,10,6);
      avPath = new TextField(getSetting(AV_COMMAND),15);
      gridbag.setConstraints(avPath,constraints);
      panel.add(avPath);
      constraints.gridwidth  = GridBagConstraints.RELATIVE;
      label = new Label(getSetting(SETTDLG_ADDPARA));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.gridwidth  = GridBagConstraints.REMAINDER;
      avParam = new TextField(getSetting(AV_ADDPARAM),15);
      gridbag.setConstraints(avParam,constraints);
      panel.add(avParam);
      constraints.insets     = new Insets(0,12,10,6);
      constraints.gridwidth  = 1;
      label = new Label(getSetting(SETTDLG_PREFIX));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.insets     = new Insets(0,0,10,6);
      avPrefix = new TextField(getSetting(AV_CLASSPATHPREFIX),15);
      gridbag.setConstraints(avPrefix,constraints);
      panel.add(avPrefix);
      constraints.gridwidth  = GridBagConstraints.REMAINDER;
      avSpaces = new Checkbox(getSetting(SETTDLG_SPACES),
                   (new Boolean(getSetting(AV_TRAILINGSPACE))).booleanValue());
      gridbag.setConstraints(avSpaces,constraints);
      panel.add(avSpaces);
      constraints.insets     = new Insets(0,12,10,6);
      constraints.gridwidth  = 1;
      avUseCP = new Checkbox(getSetting(SETTDLG_USEPATH),
                  (new Boolean(getSetting(AV_USECLASSPATH))).booleanValue());
      gridbag.setConstraints(avUseCP,constraints);
      panel.add(avUseCP);
      // insert a dummy label to get some extra spacing
      constraints.insets     = new Insets(0,0,10,6);
      label = new Label("");
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.gridwidth  = GridBagConstraints.REMAINDER;
      avMulti = new Checkbox(getSetting(SETTDLG_MULTI),
                  (new Boolean(getSetting(AV_MULTIINSTANCE))).booleanValue());
      gridbag.setConstraints(avMulti,constraints);
      panel.add(avMulti);
      constraints.insets     = new Insets(0,0,10,0);
      constraints.fill       = GridBagConstraints.HORIZONTAL;
      separator = new SeparatorCanvas();
      gridbag.setConstraints(separator,constraints);
      panel.add(separator);
      // HTML browser settings
      constraints.insets     = new Insets(0,6,10,6);
      constraints.fill       = GridBagConstraints.NONE;
      label = new Label(getSetting(SETTDLG_BRTITLE));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.insets     = new Insets(0,12,10,6);
      constraints.gridwidth  = 1;
      label = new Label(getSetting(SETTDLG_PATH));
      gridbag.setConstraints(label,constraints);
      panel.add(label);

      Panel p = new Panel(new FlowLayout(FlowLayout.LEFT,0,0));
      Button b = new Button("^");

      constraints.insets     = new Insets(0,0,10,6);
      brPath = new TextField(getSetting(BR_COMMAND),12);
      p.add(brPath);
      b.addActionListener(this);
      p.add(b);
      gridbag.setConstraints(p,constraints);
      panel.add(p);
      constraints.gridwidth  = GridBagConstraints.RELATIVE;
      label = new Label(getSetting(SETTDLG_ADDPARA));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.gridwidth  = GridBagConstraints.REMAINDER;
      brParam = new TextField(getSetting(BR_ADDPARAM),15);
      gridbag.setConstraints(brParam,constraints);
      panel.add(brParam);
      constraints.insets     = new Insets(0,12,10,6);
      constraints.gridwidth  = 1;
      label = new Label(getSetting(SETTDLG_PREFIX));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.insets     = new Insets(0,0,10,6);
      brPrefix = new TextField(getSetting(BR_CLASSPATHPREFIX),15);
      gridbag.setConstraints(brPrefix,constraints);
      panel.add(brPrefix);
      constraints.gridwidth  = GridBagConstraints.REMAINDER;
      brSpaces = new Checkbox(getSetting(SETTDLG_SPACES),
                   (new Boolean(getSetting(BR_TRAILINGSPACE))).booleanValue());
      gridbag.setConstraints(brSpaces,constraints);
      panel.add(brSpaces);
      constraints.insets     = new Insets(0,12,10,6);
      constraints.gridwidth  = 1;
      constraints.gridheight = GridBagConstraints.RELATIVE;
      brUseCP = new Checkbox(getSetting(SETTDLG_USEPATH),
                  (new Boolean(getSetting(BR_USECLASSPATH))).booleanValue());
      gridbag.setConstraints(brUseCP,constraints);
      panel.add(brUseCP);
      // insert a dummy label to get some extra spacing
      constraints.insets     = new Insets(0,0,10,6);
      label = new Label("");
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.gridwidth  = GridBagConstraints.REMAINDER;
      brMulti = new Checkbox(getSetting(SETTDLG_MULTI),
                  (new Boolean(getSetting(BR_MULTIINSTANCE))).booleanValue());
      gridbag.setConstraints(brMulti,constraints);
      panel.add(brMulti);
      constraints.insets     = new Insets(0,0,10,0);
      constraints.gridheight = GridBagConstraints.REMAINDER;
      constraints.fill       = GridBagConstraints.HORIZONTAL;
      separator = new SeparatorCanvas();
      gridbag.setConstraints(separator,constraints);
      panel.add(separator);
      pack();
      center(parent);
      setResizable(false);
    } // end constructor SettingsDlg

    /** Write the dialog controls content to the project properties. */
    public void storeResults() {
      project.put(SELECTED_VIEWER,"" + choice.getSelectedIndex());
      project.put(VM_COMMAND,vmPath.getText());
      project.put(VM_ADDPARAM,vmParam.getText());
      project.put(VM_USECLASSPATH,"" + vmUseCP.getState());
      project.put(VM_CLASSPATHPREFIX,vmPrefix.getText());
      project.put(VM_TRAILINGSPACE,"" + vmSpaces.getState());
      project.put(VM_MULTIINSTANCE,"" + vmMulti.getState());
      project.put(AV_COMMAND,avPath.getText());
      project.put(AV_ADDPARAM,avParam.getText());
      project.put(AV_USECLASSPATH,"" + avUseCP.getState());
      project.put(AV_CLASSPATHPREFIX,avPrefix.getText());
      project.put(AV_TRAILINGSPACE,"" + avSpaces.getState());
      project.put(AV_MULTIINSTANCE,"" + avMulti.getState());
      project.put(BR_COMMAND,brPath.getText());
      project.put(BR_ADDPARAM,brParam.getText());
      project.put(BR_USECLASSPATH,"" + brUseCP.getState());
      project.put(BR_CLASSPATHPREFIX,brPrefix.getText());
      project.put(BR_TRAILINGSPACE,"" + brSpaces.getState());
      project.put(BR_MULTIINSTANCE,"" + brMulti.getState());
    } // end storeResults

    // --- ActionListener ---------------------------------------------
    /** Invoked when an action occurs. */
    public void actionPerformed(ActionEvent e) {
      File file = new File(brPath.getText());
      String name,path;
      FileDialog dlg;
      File result = null;

      dlg = new FileDialog(parent);
      dlg.setDirectory(file.getParent());
      dlg.setFile(file.getName());
      dlg.show();
      name = dlg.getFile();
      if (name != null) {
        path = dlg.getDirectory();
        if (path == null) result = new File(name);
          else result = new File(path,name);
      } // end if
      if (result == null) return;
      brPath.setText(result.getAbsolutePath());
    } // end actionPerformed

  } // end SettingsDlg

} // end DefaultExecute

/* ******************************************************************
   end of file
*********************************************************************/
