/* ******************************************************************

   class    : RootDlg
   date     : Feb 18 1999
   compiler : javac (1.2)

******************************************************************* */

package Dialogs;

/* ******************************************************************
   import statements
******************************************************************* */

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.ResourceBundle;
import java.util.Properties;

import CustCtrl.BaseDialog;
import Utils.ResourceUtil;

/**
 * A class that lets you specify the root of the file system.
 *
 * @author puehringer edgar
 */
public class RootDlg extends BaseDialog {

  // class variable declarations
  private static final String TITLE   = "fileRootDlg.title";
  private static final String BUTTONS = "fileRootDlg.buttons";
  private static final String TEXT    = "fileRootDlg.text";

  // instance variable declarations
  /** The choice displaying the file system roots. */
  protected Choice choice;
  /** The file system roots. */
  protected File[] files;

  // constructors
  /** Create a new RootDlg object. */
  public RootDlg(Frame parent, ResourceBundle resources) {
    super(parent,true);

    Panel panel = getContentPanel();

    setTitle(ResourceUtil.getResStringOrKey(resources,TITLE));
    createButtons(ResourceUtil.getResStringOrKey(resources,BUTTONS));
    panel.add(new Label(ResourceUtil.getResStringOrKey(resources,TEXT)));
    files  = File.listRoots();
    choice = new Choice();
    for (int i=0; i<files.length; i++) choice.add("" + files[i]);
    panel.add(choice);
    pack();
    center(parent);
    setResizable(false);
  } // end constructor RootDlg

  /** Create a new RootDlg object. */
  public RootDlg(Frame parent, ResourceBundle resources, File curRoot) {
    this(parent,resources);
    if (curRoot == null) return;

    String str = curRoot.getAbsolutePath();

    for (int i=0; i<files.length; i++) {
      if (str.startsWith("" + files[i])) {
        choice.select(i);
        return;
      } // end if
    } // end for
  } // end constructor RootDlg

  // methods
  /** Read the TextFields text. */
  public File getRoot() {
    int index = choice.getSelectedIndex();

    if (index < 0) return null;
    return files[index];
  } // end getResult

} // end RootDlg

/* ******************************************************************
   end of file
******************************************************************* */
