/* ******************************************************************

   class    : SysInfoDlg
   date     : 07.12.97
   compiler : javac (jdk 1.1.6)
   modified : 03.02.98 - auto cancel mode for applet operation added
              10.08.98 - auto cancel mode removed, added to package
                         Dialogs, now extends BaseDialog;
                         New constructor replaces all others.

*********************************************************************/

package Dialogs;

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.util.ResourceBundle;

import CustCtrl.BaseDialog;
import CustCtrl.InsetsPanel;
import Utils.ResourceUtil;

/**
 *
 * A dialog that displays system informations.
 *
 * @author puehringer edgar
 *
 */
public class SysInfoDlg extends BaseDialog implements ActionListener {

  // class variable declarations
  private final static String TITLE   = "title";
  private final static String BUTTONS = "buttons";
  private final static String[] properties = {
    "java.version",
    "java.vendor",
    "java.vendor.url",
    "java.class.version",
    "java.compiler",
    "java.home",
    "os.name",
    "os.arch",
    "os.version",
    "file.separator",
    "path.separator"
  };

  // instance variable declarations
  private Label percentLabel;
  private int freePercent;

  // constructors
  /** Create a new SysInfoDlg object. */
  public SysInfoDlg(Frame parent, ResourceBundle resources,
                    String resourcePrefix) {
    super(parent,true);

    Panel panel = new InsetsPanel(new Insets(6,10,6,10));
    String str;

    getContentPanel().add(panel);
    setTitle(ResourceUtil.getResStringOrKey(resources,
                    resourcePrefix + TITLE));
    createButtons(ResourceUtil.getResStringOrKey(resources,
                    resourcePrefix + BUTTONS));
    addActionListener(this);
    panel.setLayout(new GridLayout(properties.length + 2,2));
    for (int i=0; i<properties.length; i++) {
      panel.add(new Label(properties[i]));
      str = System.getProperty(properties[i]);
      if (str == null) str = "Not specified";
      panel.add(new Label(str));
    } // end for
    Runtime r = Runtime.getRuntime();
    panel.add(new Label("total.memory"));
    panel.add(new Label(r.totalMemory()+" Bytes"));
    panel.add(new Label("free.memory"));
    updateFreePercent(r);
    percentLabel = new Label(freePercent + " %    ");
    panel.add(percentLabel);
    pack();
    center(parent);
    setResizable(false);
  } // end constructor SysInfoDlg

  // class methods
    /** Only for debuging purposes. */
  public static void main(String[] args) throws java.util.MissingResourceException {
    Frame frame = new Frame();
    SysInfoDlg dlg;
    ResourceBundle res;

    if (args.length < 1) {
      System.out.println("Resource file must be specified.");
      System.exit(1);
    } // end if
    res = ResourceBundle.getBundle(args[0],java.util.Locale.getDefault());
    dlg = new SysInfoDlg(frame,res,"sysInfoDlg.");
    dlg.show();
    System.out.println("isCanceled returned " + dlg.isCanceled());
    System.exit(0);
  } // end main

  // methods
  private void updateFreePercent(Runtime r) {
    if (r == null) r = Runtime.getRuntime();
    freePercent = (int)(100 * r.freeMemory() / r.totalMemory());
  } // end updatePercent

  /**
   * Called to close this dialog.
   */
  protected void closeDialog(AWTEvent event) {
    if ((event instanceof WindowEvent) || !isCanceled()) {
      super.closeDialog(event);
    } // end if
  } // end closeDialog

  public void actionPerformed(ActionEvent e) {
    String cmd   = e.getActionCommand();
    String label;

    if (buttonVector.size() < 2) return;
    label = ((Button)buttonVector.elementAt(1)).getLabel();
    if (cmd.equals(label)) { // if it's the collect button
      System.gc();
      updateFreePercent(null);
      percentLabel.setText(freePercent+" %    ");
    } // end if
  } // end actionPerformed

} // end SysInfoDlg

/* ******************************************************************
   end of file
*********************************************************************/
