package ErrorParser;
import java.util.*;

// changed PE 30.07.98
public class Parser {
// end changed PE 30.07.98
	private static final int maxT = 14;
	private static final int maxP = 14;

	private static final boolean T = true;
	private static final boolean x = false;
	private static final int minErrDist = 2;
	private static int errDist = minErrDist;

	static Token token;   // last recognized token
	static Token t;       // lookahead token

	private static boolean debug = false;
  private static ParserObservable observable;

  /** For debugging purposes. */
  public static void main(String[] args) {
    debug = true;
    if (args.length != 1) {
      System.out.println("ErrorParser.Parser: Wrong number of parameters.");
      return;
    } // end if
    try {
      java.io.File file          = new java.io.File(args[0]);
      java.io.FileInputStream is = new java.io.FileInputStream(file);

      parse(is);
      is.close();
    } catch (java.io.IOException excep) {
      excep.printStackTrace();
    }
  } // end main

  /** Initializes the scanner and parses the given stream. */
  public static void parse(java.io.InputStream is) {
    java.io.PushbackInputStream pis = new java.io.PushbackInputStream(is);
    int ch;

    try {
      do {
        ch = pis.read();
      } while ((ch == (int)' ') ||
               (ch == (int)'\t') ||
               (ch == (int)'\n') ||
               (ch == (int)'\f') ||
               (ch == (int)'\r'));
    } catch (java.io.IOException excep) {
      excep.printStackTrace();
      ch = -1;
    }
    // This is, because Coco don't like empty streams
    if (ch == -1) {
      reportEnd(0,0);
    }
    else {
      try {
        pis.unread(ch);
        Scanner.Init(pis);
        Parse();
      } catch (java.io.IOException excep) {
        excep.printStackTrace();
        reportEnd(0,0);
      }
    } // end if
  } // end parse

  /** Set a Observer for the parser to notify decoded error messages. */
  public static void setObservable(ParserObservable obs) {
    observable = obs;
  } // end setObservable

  /** Called by the parser to report a decoded error message. */
  private static void reportError(ErrorDesc errorDesc) {
    if (debug) {
      System.out.println("" + errorDesc);
    }
    else {
      observable.notify(errorDesc);
    } // end if
  } // end reportError

  /** Called by the parser to report the en of the decoding process. */
  private static void reportEnd(int errors, int warnings) {
    if (debug) {
      System.out.println(errors + " errors, " + warnings + " warnings");
    }
    else {
    int[] aInt = new int[2];

    aInt[0] = errors;
    aInt[1] = warnings;
    observable.notify(aInt);
    } // end if
  } // end reportEnd

  /** Converts a token to an integer, redirects exceptions to the ErrorStream. */
  private static int tokenToInt(Token tok) {
    int i = -1;

    try {
      i = Integer.valueOf(tok.val).intValue();
    } catch (NumberFormatException excep) {
      Scanner.err.StoreError(101, tok.line, tok.col,"number format exception");
    }
    return i;
  } // end tokenToInt



	static void Error(int n) {
		if (errDist >= minErrDist) Scanner.err.ParsErr(n, t.line, t.col);
		errDist = 0;
	}

	static void SemError(int n) {
		if (errDist >= minErrDist) Scanner.err.SemErr(n, token.line, token.col);
		errDist = 0;
	}

	static boolean Successful() {
		return Scanner.err.count == 0;
	}

	static String LexString() {
		return token.str;
	}

	static String LexName() {
		return token.val;
	}

	static String LookAheadString() {
		return t.str;
	}

	static String LookAheadName() {
		return t.val;
	}

	private static void Get() {
		for (;;) {
			token = t;
			t = Scanner.Scan();
			if (t.kind <= maxT) {errDist++; return;}

			t = token;
		}
	}

	private static void Expect(int n) {
		if (t.kind == n) Get(); else Error(n);
	}

	private static boolean StartOf(int s) {
		return set[s][t.kind];
	}

	private static void ExpectWeak(int n, int follow) {
		if (t.kind == n) Get();
		else {
			Error(n);
			while (!StartOf(follow)) Get();
		}
	}

	private static boolean WeakSeparator(int n, int syFol, int repFol) {
		boolean[] s = new boolean[maxT+1];
		if (t.kind == n) {Get(); return true;}
		else if (StartOf(repFol)) return false;
		else {
			for (int i = 0; i <= maxT; i++) {
				s[i] = set[syFol][i] || set[repFol][i] || set[0][i];
			}
			Error(n);
			while (!s[t.kind]) Get();
			return StartOf(syFol);
		}
	}

	private static String AllOne() {
		String text;
		switch (t.kind) {
		case 1: {
			Get();
			break;
		}
		case 2: {
			Get();
			break;
		}
		case 4: {
			Get();
			break;
		}
		case 5: {
			Get();
			break;
		}
		case 6: {
			Get();
			break;
		}
		case 7: {
			Get();
			break;
		}
		case 9: {
			Get();
			break;
		}
		case 10: {
			Get();
			break;
		}
		case 11: {
			Get();
			break;
		}
		case 12: {
			Get();
			break;
		}
		case 13: {
			Get();
			break;
		}
		default: Error(15);
		}
		text = token.val;
		return text;
	}

	private static void WarningRest() {
		if (t.kind == 12) {
			Get();
		} else if (t.kind == 13) {
			Get();
		} else Error(16);
	}

	private static int SummaryTwo() {
		int warnings;
		Expect(6);
		Expect(2);
		warnings = tokenToInt(token);
		WarningRest();
		return warnings;
	}

	private static Object SourceFile() {
		Object file;
		String name;
		Expect(1);
		name = token.val;
		if (t.kind == 5) {
			Get();
			name += token.val;
			Expect(1);
			name += token.val;
		}
		Expect(4);
		name += token.val;
		Expect(9);
		name += token.val;
		file = new java.io.File(name);
		
		return file;
	}

	private static void ErrorPoint() {
		Expect(10);
		Expect(3);
	}

	private static String SourceLine() {
		String code;
		code = AllMore();
		return code;
	}

	private static String AllMore() {
		String text;
		String first,next;
		first = AllOne();
		text = first;
		while (StartOf(1)) {
			next = AllOne();
			text += next;
		}
		Expect(3);
		return text;
	}

	private static void LongWarning(ErrorDesc ed) {
		String msg,code;

		ed.type = ErrorDesc.LONG_WARNING;
		
		Expect(8);
		Expect(5);
		msg = AllMore();
		ed.msg = msg;
		code = SourceLine();
		ed.code = code;
		ErrorPoint();
	}

	private static void LongError(ErrorDesc ed) {
		String msg, code;

		ed.type = ErrorDesc.LONG_ERROR;
		
		msg = AllMore();
		ed.msg = msg;
		code = SourceLine();
		ed.code = code;
		ErrorPoint();
	}

	private static void FilePlusLine(ErrorDesc ed) {
		Object file;
		file = SourceFile();
		ed.file = (java.io.File)file;
		Expect(5);
		Expect(2);
		ed.line = tokenToInt(token);
		Expect(5);
	}

	private static void ShortWarning(ErrorDesc ed) {
		String msg;

		ed.type = ErrorDesc.SHORT_WARNING;
		
		Expect(8);
		Expect(5);
		msg = AllMore();
		ed.msg = msg;
	}

	private static void ShortError(ErrorDesc ed) {
		String msg;

		ed.type = ErrorDesc.SHORT_ERROR;
		
		Expect(7);
		Expect(5);
		msg = AllMore();
		ed.msg = msg;
	}

	private static void LongMsg(ErrorDesc ed) {
		FilePlusLine(ed);
		if (StartOf(1)) {
			LongError(ed);
		} else if (t.kind == 8) {
			LongWarning(ed);
		} else Error(17);
	}

	private static void ShortMsg(ErrorDesc ed) {
		ed.line = -1;
		ed.code = null;
		
		if (t.kind == 7) {
			ShortError(ed);
		} else if (t.kind == 8) {
			ShortWarning(ed);
		} else Error(18);
	}

	private static void Summary() {
		int errors   = 0;
		int warnings = 0;
		int tmp;
		
		Expect(2);
		tmp = tokenToInt(token);
		if (t.kind == 7) {
			Get();
			errors = tmp;
			if (t.kind == 6) {
				warnings = SummaryTwo();
			}
		} else if (t.kind == 11) {
			Get();
			errors = tmp;
			if (t.kind == 6) {
				warnings = SummaryTwo();
			}
		} else if (t.kind == 12 || t.kind == 13) {
			WarningRest();
			warnings = tmp;
		} else Error(19);
		reportEnd(errors,warnings);
	}

	private static ErrorDesc Message() {
		ErrorDesc ed;
		ed = new ErrorDesc();
		if (t.kind == 7 || t.kind == 8) {
			ShortMsg(ed);
		} else if (t.kind == 1) {
			LongMsg(ed);
		} else Error(20);
		return ed;
	}

	private static void ErrorParser() {
		ErrorDesc errorDesc;
		errorDesc = Message();
		reportError(errorDesc);
		while (t.kind == 1 || t.kind == 7 || t.kind == 8) {
			errorDesc = Message();
			reportError(errorDesc);
		}
		Summary();
	}



	static void Parse() {
		t = new Token();
		Get();
		ErrorParser();

	}

	private static boolean[][] set = {
	{T,x,x,x, x,x,x,x, x,x,x,x, x,x,x,x},
	{x,T,T,x, T,T,T,T, x,T,T,T, T,T,x,x}

	};
}
