package ErrorParser;
import java.io.*;
import java.util.*;

class Token {
	int kind;    // token kind
	int pos;     // token position in the source text (starting at 0)
	int col;     // token column (starting at 0)
	int line;    // token line (starting at 1)
	String str;  // exact string value
	String val;  // token string value (uppercase if ignoreCase)
}

class Buffer {

// Portability - use the following for Java 1.0
//	static byte[] buf;  // Java 1.0
// Portability - use the following for Java 1.1
//	static char[] buf;  // Java 1.1

	static char[] buf;  // Java 1.1

	static int bufLen;
	static int pos;
	static final int eof = 65535;
// changed PE 30.07.98
  static boolean operateOnStream = false;
  static Reader in;

  static void Fill(InputStream is) {
    in              = new BufferedReader(new InputStreamReader(is));
    operateOnStream = true;
  } // end Fill
// end changed PE 30.07.98

	static void Fill(String name) {
// changed PE 30.07.98
    operateOnStream = false;
// end changed PE 30.07.98
		try {
			File f = new File(name); bufLen = (int) f.length();

// Portability - use the following for Java 1.0
//			BufferedInputStream s = new BufferedInputStream(new FileInputStream(f), bufLen);
//			buf = new byte[bufLen];  // Java 1.0
// Portability - use the following for Java 1.1
//			BufferedReader s = new BufferedReader(new FileReader(f), bufLen);
//			buf = new char[bufLen];  // Java 1.1

			BufferedReader s = new BufferedReader(new FileReader(f), bufLen);
			buf = new char[bufLen];  // Java 1.1

			int n = s.read(buf); pos = 0;
		} catch (IOException e) {
			System.out.println("--- cannot open file " + name);
			System.exit(0);
		}
	}

// changed PE 30.07.98
//  static void Set(int position) {
//    if (position < 0) position = 0; else if (position >= bufLen) position = bufLen;
//    pos = position;
//  }
// end changed PE 30.07.98

	static int read() {
// changed PE 30.07.98
    if (operateOnStream) {
      int i;

      if (in == null) return eof;
      try {
        i = in.read();
      } catch (IOException excep) {
        excep.printStackTrace();
        i = -1;
      }
      if (i == -1) {
        try {
          in.close();
        } catch (IOException excep) {
          excep.printStackTrace();
        }
        in = null;
        i  = eof;
      } // end if
      return i;
    }
    else{
      if (pos < bufLen) return (int) buf[pos++]; else return eof;
    }
	}
// end changed PE 30.07.98
}

class Scanner {

	static ErrorStream err;  // error messages

	private static final char EOF = '\0';
	private static final char CR  = '\r';
	private static final char LF  = '\n';
	private static final int noSym = 14;
	private static final int[] start = {
	  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  4,  1,  1,  3,  1,  1,
	  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  7,  1,  5,  1,
	  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  6,  1,  1,  1,  1,  1,
	  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	  0};

	private static Token t;        // current token
	private static char strCh;     // current input character (original)
	private static char ch;        // current input character (for token)
	private static char lastCh;    // last input character
	private static int pos;        // position of current character
	private static int line;       // line number of current character
	private static int lineStart;  // start position of current line
	private static BitSet ignore;  // set of characters to be ignored by the scanner
	private static int offset = 0; // 1 - MsDos, 0 - Unix/Mac
// changed PE 30.07.98
  private static InputStream inStream;

  static void Init(InputStream is, ErrorStream e) {
    String file = null;

    inStream = is;
    Init(file,e);
  } // end Init

  static void Init(InputStream is) {
    Init(is,new ErrorStream());
  } // end Init
// end changed PE 30.07.98

	static void Init (String file, ErrorStream e) {
		ignore = new BitSet(128);
		ignore.set(9); ignore.set(12); ignore.set(32); 
		err = e;
// changed PE 30.07.98
    if (file == null) Buffer.Fill(inStream);
      else Buffer.Fill(file);
// end changed PE 30.07.98
		pos = -1; line = 1; lineStart = 0; lastCh = 0;
		NextCh();
	}

	static void Init (String file) {
		Init(file, new ErrorStream());
	}

	private static void NextCh() {
		lastCh = ch;
		strCh = (char) Buffer.read(); pos++;
		ch = strCh;
		if (ch == LF && lastCh == CR) offset = 1; // MS-Dos format
		if ((ch == CR) || (ch == LF) && (lastCh != CR)) {line++; lineStart = pos + 1;}
		if (ch > '\u007f') {
			if (ch == '\uffff') ch = EOF;
			else {
				Scanner.err.SemErr(-1, line, (pos + 1 - lineStart - offset));
				ch = ' ';
			}
		}
	}

	private static boolean Comment() {
		int level, line0, lineStart0; char startCh;
		level = 1; line0 = line; lineStart0 = lineStart;

		return false;
	}

	private static void CheckLiteral(StringBuffer buf) {
		t.val = buf.toString();
		switch (t.val.charAt(0)) {
			case 'N': {
				if (t.val.equals("Note")) t.kind = 8;
				break;}
			case '^': {
				if (t.val.equals("^")) t.kind = 10;
				break;}
			case 'e': {
				if (t.val.equals("error")) t.kind = 7;
				else if (t.val.equals("errors")) t.kind = 11;
				break;}
			case 'j': {
				if (t.val.equals("java")) t.kind = 9;
				break;}
			case 'w': {
				if (t.val.equals("warning")) t.kind = 12;
				else if (t.val.equals("warnings")) t.kind = 13;
				break;}
		}
	}

	static Token Scan() {
		int state, apx;
		StringBuffer buf;
		while (ignore.get((int)ch)) NextCh();

		t = new Token();
		t.pos = pos; t.col = pos - lineStart + 1 - offset; t.line = line;
		buf = new StringBuffer();
		state = start[ch];
		apx = 0;
		loop: for (;;) {
			buf.append(strCh);
			NextCh();
			switch (state) {
				case 0:
					{t.kind = noSym; break loop;} // NextCh already done
				case 1:
					if (ch <= 9
					  || ch >= 11 && ch <= 12
					  || ch >= 14 && ch <= '+'
					  || ch == '-'
					  || ch >= '/' && ch <= '9'
					  || ch >= ';') {break;}
					else {t.kind = 1; CheckLiteral(buf); break loop;}
				case 2:
					if (ch >= '0' && ch <= '9') {break;}
					else {t.kind = 2; break loop;}
				case 3:
					if (ch == 10) {break;}
					else {t.kind = 3; break loop;}
				case 4:
					{t.kind = 3; break loop;}
				case 5:
					{t.kind = 4; break loop;}
				case 6:
					{t.kind = 5; break loop;}
				case 7:
					{t.kind = 6; break loop;}
				case 8:
					{t.kind = 0; break loop;}
			}
		}
		t.str = buf.toString();
		t.val = t.str;
		return t;
	}
}
