/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : EventPoster
   date       : 19.05.98
   description: a class that posts an awt event in periodical

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class EventPoster extends Thread {

  // class variable declarations
  public final static float DEF_ERROR    = 0.1f;
  public final static int   DEF_PRIORITY = (Thread.MIN_PRIORITY +
                                            Thread.NORM_PRIORITY) / 2;

  // instance variable declarations
  private AWTEvent theEvent;
  private long millis;
  private float maxError;
  private boolean enabled;
  private long oldTime;
  private long incTime;

  // constructors
  /**
   * Create a initially enabled event poster.
   *
   * @param theEvent The awt event to post.
   * @param millis The period of time between two events
   */
  public EventPoster(AWTEvent theEvent, long millis) {
    this(theEvent,millis,DEF_ERROR,true);
  } // end constructor EventPoster

  /**
   * Create a new event poster.
   *
   * @param theEvent The awt event to post.
   * @param millis The period of time between two events
   * @param enabled True, if events should be posted.
   */
  public EventPoster(AWTEvent theEvent, long millis, boolean enabled) {
    this(theEvent,millis,DEF_ERROR,enabled);
  } // end constructor EventPoster

  /**
   * Create a initially enabled event poster.
   *
   * @param theEvent The awt event to post.
   * @param millis The period of time between two events
   * @param maxError The maximal time period error between 0 and 1.
   */
  public EventPoster(AWTEvent theEvent, long millis, float maxError) {
    this(theEvent,millis,maxError,true);
  } // end constructor EventPoster

  /**
   * Create a new event poster.
   *
   * @param theEvent The awt event to post.
   * @param millis The period of time between two events
   * @param maxError The maximal time period error between 0 and 1.
   * @param enabled True, if events should be posted.
   */
  public EventPoster(AWTEvent theEvent, long millis, float maxError,
                     boolean enabled) {
    this.theEvent = theEvent;
    this.millis   = millis;
    this.maxError = maxError;
    this.enabled  = enabled;
    oldTime       = System.currentTimeMillis();
    incTime       = (long)(this.millis * this.maxError);
    setPriority(DEF_PRIORITY);
  } // end constructor EventPoster

  // instance methods
  /**
   * Enables or disables this event poster, depending on the value of
   * the parameter b.
   */
  public synchronized void setEnabled(boolean b) {
    oldTime = System.currentTimeMillis();
    enabled = b;
  } // end setEnabled

  /**
   * Determines whether this event poster is enabled.
   */
  public synchronized boolean isEnabled() {
    return enabled;
  } // end isEnabled

  /**
   * Set the time period in milliseconds.
   */
  public synchronized void setMillis(long millis) {
    this.millis = millis;
  } // end setMillis

  /**
   * Return the time period in milliseconds.
   */
  public synchronized long getMillis() {
    return millis;
  } // end getMillis

  /**
   * Set the time counter to zero, so that a new time periode starts.
   */
  public synchronized void resetTimer() {
    oldTime = System.currentTimeMillis();
  } // end resetTimer

  private synchronized long getOldTime() {
    return oldTime;
  } // end getOldTime

  //--- Thread ------------------------------------

  public void run() {
    while (true) {
      try {
        Thread.currentThread().sleep(15000);
      } catch (InterruptedException e) {}
      if (isEnabled()) {
        long curTime = System.currentTimeMillis();
        if ((getOldTime() + getMillis()) < curTime) {
          EventQueue eq = Toolkit.getDefaultToolkit().getSystemEventQueue();
          eq.postEvent(theEvent);
          resetTimer();
        } // end if
      } // end if
    } // end while
  } // end run

} // end EventPoster

/* ******************************************************************
   end of file
*********************************************************************/
