/* ******************************************************************

   class    : MenuTestFrame
   date     : Sep 27 1998
   compiler : javac (1.1.6)
   modified : 11.10.98 - AWT toolbars are now possible.

******************************************************************* */

package MenuPreview;

/* ******************************************************************
   import statements
******************************************************************* */

import java.awt.*;
import java.awt.event.*;
import java.io.InputStream;
import java.io.IOException;
import java.util.*;

/**
 * A class that displays a Frame with a menu generated from an
 * input stream.
 *
 * @author puehringer edgar
 */
public class MenuTestFrame extends Frame
  implements ActionListener {

  // instance variable declarations
  private TextArea editor;

  // constructors
  /** Create a new MenuTestFrame object. */
  public MenuTestFrame(String title, Image iconImg, InputStream is,
                       String menuKey, String toolBarKey) throws IOException {
    super(title);

    Properties prop;
    MenuLoader menuLoader;
    MenuBar menuBar;
    ToolBarLoader toolBarLoader;
    Panel toolBar;
    boolean done = false;

    try {
      if (iconImg != null) setIconImage(iconImg);
      addWindowListener(
        new WindowAdapter() {

          public void windowClosing(WindowEvent e) {
            setVisible(false);
            dispose();
          } // end windowClosing

        }
      );
      prop = new Properties();
      prop.load(is);
      setLayout(new BorderLayout());
      editor = new TextArea();
      editor.setEditable(false);
      add("Center",editor);
      if ((toolBarKey != null) && !toolBarKey.equals("")) {
        toolBarLoader = new ToolBarLoader(prop) {

          protected void reportError(String text) {
            writeToEditor("ToolBarLoader: " + text + ".");
          } // end reportError

        };
        toolBarLoader.setDefaultActionListener(this);
        toolBar = toolBarLoader.createToolBar(toolBarKey);
        if (toolBar != null) add("North",toolBar);
      } // end if
      if ((menuKey != null) && !menuKey.equals("")) {
        menuLoader = new MenuLoader(prop) {

          protected void reportError(String text) {
            writeToEditor("MenuLoader: " + text + ".");
          } // end reportError

        };
        menuLoader.setDefaultActionListener(this);
        menuBar = menuLoader.createMenuBar(menuKey);
        if (menuBar != null) setMenuBar(menuBar);
      } // end if
      done = true;
    } finally {
      if (!done) dispose();
    }
  } // end constructor MenuTestFrame

  /** Insert a string into the embedded editor. */
  protected void writeToEditor(String text) {
    int start = editor.getSelectionStart();
    int end   = editor.getSelectionEnd();

    editor.setEditable(true);
    editor.replaceRange(text + "\n",start,end);
    editor.setEditable(false);
  } // end writeToError

  // methods
  // --- ActionListener ---------------------------------------------
  /** Invoked when an action occurs. */
  public void actionPerformed(ActionEvent e) {
    Object obj = e.getSource();
    String str = e.getActionCommand();

    if (obj instanceof CheckboxMenuItem) {
      if (((CheckboxMenuItem)obj).getState()) str += " selected";
        else str += " unselected";
    } // end if
    writeToEditor(str);
  } // end actionPerformed

} // end MenuTestFrame

/* ******************************************************************
   end of file
******************************************************************* */
