/* ******************************************************************

   class    : LocatableErrorMessage
   date     : 24.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

package Messages;

/* ******************************************************************
   import statements
*********************************************************************/

import java.io.File;

/**
 * A error message that can be assigned to a location in a
 * source file.
 *
 * @author puehringer edgar
 */
public class LocatableErrorMessage extends ErrorMessage {

  // instance variable declarations
  /** The path to the file field's home directory. */
  public File path;
  /** The source file containing the error. */
  public File file;
  /** The line in the source file, where the error occured. */
  public int line;

  // constructors
  /** Create a new LocatableErrorMessage object. */
  public LocatableErrorMessage(int type, String msg, File file,
                               int line, String code) {
    this(type,msg,null,file,line,code);
  } // end constructor LocatableErrorMessage

  /** Create a new LocatableErrorMessage object. */
  public LocatableErrorMessage(int type, String msg, File path,
                               File file, int line, String code) {
    super(type,msg,code);
    this.path = path;
    this.file = file;
    this.line = line;
  } // end constructor LocatableErrorMessage

  // methods
  /** Returns a description of this message. */
  public String getDescription() {
    String str = file.getAbsolutePath();

    if (path != null) {
      String p = path.getAbsolutePath();

      if (str.startsWith(p)) {
        str = str.substring(p.length());
        if (str.startsWith(File.separator)) str = str.substring(1);
      } // end if
    } // end if
    return str + ": " + line + ": " + text;
  } // end getDescription

  /** Returns the source file containing the error. */
  public File getFile() {
    return file;
  } // end getFile

  /** Returns the content of the line, where the error occured. */
  public String getCode() {
    return getPopupInfo();
  } // end getCode

} // end LocatableErrorMessage

/* ******************************************************************
   end of file
*********************************************************************/
