/* ******************************************************************

   class    : RunAction
   date     : 22.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.util.Observable;

import Messages.LockMessage;
import Messages.UnlockMessage;
import Messages.TerminateMessage;

/**
 * Run a executable file.  All source files, that have been changed
 * will be compiled before.
 *
 * @author puehringer edgar
 */
class RunAction extends CompileAction {

  // class variable declarations
  private static final String RUN_ACTION = "run";

  // instance varaible declarations
  protected AbstractLinker linker;
  protected AbstractExecute execute;
  protected boolean success;

  // constructors
  RunAction(Frame frame, AbstractCompiler compiler,
            AbstractLinker linker, AbstractExecute execute) {
    super(RUN_ACTION,frame,compiler);
    this.linker  = linker;
    this.execute = execute;
    setEnabled((this.linker != null) && (this.execute != null));
  } // end constructor RunAction

  RunAction(String nm, Frame frame, AbstractCompiler compiler,
            AbstractLinker linker, AbstractExecute execute) {
    super(nm,frame,compiler);
    this.linker  = linker;
    this.execute = execute;
    setEnabled((this.linker != null) && (this.execute != null));
  } // end constructor RunAction

  // methods
  public void actionPerformed(ActionEvent e) {
    if (!conditionalSave()) return;
    compiler.postMessage(new LockMessage());
    compiler.addObserver(this);
    compiler.make();
  } // end actionPerformed

  /** Set the linker command invoked by this action. */
  public void setLinker(AbstractLinker l) {
    linker = l;
    super.setEnabled((linker != null) && (execute != null));
  } // end setLinker

  /** Set the execute command invoked by this action. */
  public void setExecute(AbstractExecute e) {
    execute = e;
    super.setEnabled((linker != null) && (execute != null));
  } // end setExecute

  /** Perform the execute command. */
  protected void performExecute() {
    execute.defaultExecute(frame);
  } // end performExecute

  public void update(Observable o, Object arg) {
    if (arg instanceof TerminateMessage) {
      if (o instanceof AbstractCompiler) {
        success = ((TerminateMessage)arg).success;
        compiler.deleteObserver(this);
        if (success) {
          linker.addObserver(this);
          linker.link(frame);
          // success could be changed from a callback of the linker
          if (success) {
            execute.addObserver(this);
            performExecute();
          }
          else {
            linker.postMessage(new UnlockMessage());
          } // end if
        }
        else {
          execute.postMessage(new UnlockMessage());
        } // end if
      } else if (o instanceof AbstractLinker) {
        success = ((TerminateMessage)arg).success;
        linker.deleteObserver(this);
      } else { // if o instance of AbstractExecute
        execute.deleteObserver(this);
        execute.postMessage(new UnlockMessage());
      } // end if
    } // end if
  } // end update

} // end RunAction

/* ******************************************************************
   end of file
*********************************************************************/
