// project: SID - Structured Interactive Documentation
// author : Dipl.Ing. Stefan Chiettini

package SID.Meta;

import java.util.*;
import java.io.*;
import SID.Utils.*;

// base class for attribute representation
// 
// no resolve of names is done, output in java style order
//
public abstract class AttributeInfo extends DocumentedItem{

	protected static modifierPool modResolve;
	protected static modifierPool typeResolve;
	protected static int modMax;				// maximum number of modifiers

	private BitSet modifiers;
	
	public short type;
	public short arraydim;					// 0 if simple type, n for n-dimensional arrays

	public AttributeInfo(){
		this("", (short)-1);
	}

	public AttributeInfo(String name, short type){
		super(name);
		//this.name = name;
		this.type = type;
		modifiers = new BitSet(10);
		arraydim = 0;
	}

	public AttributeInfo(String name, String type){
		super(name);
		//this.name = name;
		this.type = (short)typeResolve.mapMod(type);
		modifiers = new BitSet(10);
		arraydim = 0;
	}

	public void setModifier(int mod){
		modifiers.set(mod);
	}

	public void removeModifier(int mod){
		modifiers.clear(mod);
	}

	public void setModifiers(BitSet mods){
		modifiers = new BitSet();
		modifiers.or(mods);
	}

	public boolean isModifierSet(int mod){
		return modifiers.get(mod);
	}

	public boolean isModifierSet(String mod){
		return modifiers.get(modResolve.mapMod(mod));
	}

	public String getType(){ return typeResolve.mapMod(type); }

	public boolean equals(AttributeInfo other){
		return (name == other.name) && (type == other.type) && (modifiers.equals(other.modifiers));
	}

	// java style output
	public String toLongString(){
		String retString = modResolve.resolveModifiers(modifiers);
		return  retString + typeResolve.mapMod(type) + " " + name;
	}

	public void load(DataInputStream in) throws IOException{
		super.load(in);
		modifiers = tool.readBitSet(in);
		name = in.readUTF();
		type = in.readShort();
		arraydim = in.readShort();
	}

	public void store(DataOutputStream out) throws IOException{
		super.store(out);
		tool.writeBitSet(modifiers, modMax, out);
		out.writeUTF(name);
		out.writeShort(type);
		out.writeShort(arraydim);
	}
}