// project: SID - Structured Interactive Documentation
// author : Dipl.Ing. Stefan Chiettini

package SID.Meta;

import java.util.*;
import java.io.*;
import java.net.*;

import SID.*;
import SID.Parser.*;
import SID.Utils.*;
//import Utils.*; /* disabled by PE */

import javax.swing.tree.*;

public final class JavaHierarchy extends Hierarchy{

  protected JavaHierarchy(ClassInfo ci){
    super(ci);
//    extension = JavaProject.javaExt; /* disdabled by PE */
    extension = ".java"; /* inserted by PE */
  }

  // filename + path must be provided
  public void scanFile(String filename){
    if (filename == null) return;

    try{
      Scanner.Init(filename, new Errors());
    }
    catch (IOException e){
      System.out.println("file '"+filename+"' not found.");
      return;
    }
    Parser.parsedClasses = new Vector(2);
    Parser.Parse();

    //File f = new File(filename);
    //long modified = f.lastModified();

    for (Enumeration e = Parser.parsedClasses.elements(); e.hasMoreElements();){
      ClassInfo ci = (ClassInfo)e.nextElement();
      ci.fileName = filename;
      addClass(ci);
    }
  }

  public void scanFile(StringReader str, URL filename){
    if (str == null) return;

    try{
      Scanner.Init(str, new Errors());
    }
    catch (IOException e){
      System.out.println("file '"+filename+"' not found.");
      return;
    }
    Parser.parsedClasses = new Vector(2);
    Parser.Parse();

    for (Enumeration e = Parser.parsedClasses.elements(); e.hasMoreElements();){
      ClassInfo ci = (ClassInfo)e.nextElement();
      ci.fileName = filename.toString();
      addClass(ci);
    }
  }

  public void scanRemoteFile(InputStream in, URL filename){
    if (in == null) return;

    try{
      Scanner.Init(in, new Errors());
    }
    catch (IOException e){
      System.out.println("file '"+filename+"' not found.");
      return;
    }
    Parser.parsedClasses = new Vector(2);
    Parser.Parse();

    //File f = new File(filename);
    //long modified = f.lastModified();

    for (Enumeration e = Parser.parsedClasses.elements(); e.hasMoreElements();){
      ClassInfo ci = (ClassInfo)e.nextElement();
      ci.fileName = filename.toString();
      //ci.lastModified = modified;
      addClass(ci);
    }
  }

  // check all classes in the hierarchy and update their sub- and superclass references
  public void connectInheritanceRelations(){
    ClassNode supClass = null;

    for (Enumeration e = classes.elements(); e.hasMoreElements();){
      ClassNode n = (ClassNode)e.nextElement();

      String cName = getClassName(n.thisClass.superName);

//      System.out.println("\n"+n.thisClass.name + " extends " + cName + " checking...");
      if (n.thisClass instanceof JavaClassInfo){

        JavaClassInfo javaClass = (JavaClassInfo)n.thisClass;
        if (cName.equals(javaClass.superName)){   // simple name found, try all imported packages
          for(Enumeration imports = javaClass.getImportList(); imports.hasMoreElements();){

            JavaImport ji = (JavaImport)imports.nextElement();
            if (ji.simpleType){           // simple class imported
//              System.out.print("  Simple Type "+ ji.importName);
              if (cName.equals(getClassName(ji.importName))){
//                System.out.print(" matches");
                supClass = (ClassNode)classes.get(new Integer(ji.importName.hashCode()));
                javaClass.superName = ji.importName;
                break;
              }
//              System.out.println("");
            }
            else{                 // whole package imported
              String fullSuperName = ji.importName + "." + cName;
//              System.out.print("  "+fullSuperName);
              supClass = (ClassNode)classes.get(new Integer(fullSuperName.hashCode()));
              if (supClass != null){
                javaClass.superName = fullSuperName;
//                System.out.print(" matches");
                break;
              }
//              System.out.println("");
            }
          }
        }
        else{
          supClass = (ClassNode)classes.get(new Integer(n.thisClass.superName.hashCode()));
        }
      }
      else supClass = (ClassNode)classes.get(new Integer(n.thisClass.superName.hashCode()));

      if (supClass != null){
        supClass.addSubClass(n);
        n.addSuperClass(supClass);
      }
    }
  }

  // packed says whether attributes and methods should be placed in separate subtrees
  protected DefaultMutableTreeNode buildClassNode(ClassInfo ci, boolean packed){
    DefaultMutableTreeNode classNode = null;

    if (ci.valid) classNode = new DefaultMutableTreeNode(ci, true);
    else return new DefaultMutableTreeNode(ci, false);

    DefaultMutableTreeNode root = classNode;
    // add class node sorted by class name
    int idx = 0;
    Enumeration children=null;
    boolean found = false;

    if (packed){
      DefaultMutableTreeNode temp = new DefaultMutableTreeNode("attributes", true);
      classNode.add(temp);
      classNode = temp;
    }

    // build up sorted subtree for the given class
    for (Enumeration e1 = ci.attributes(); e1.hasMoreElements();){
      AttributeInfo ai = (AttributeInfo)e1.nextElement();
      DefaultMutableTreeNode a = new DefaultMutableTreeNode(ai, false);
      idx = 0;
      children = classNode.children();
      found = false;

      while (children.hasMoreElements() && !found){
        DefaultMutableTreeNode cur = (DefaultMutableTreeNode)children.nextElement();
        String curName = (String)((AttributeInfo)cur.getUserObject()).name;
        if (curName.compareTo(ai.name) > 0){
          classNode.insert(a, idx);
          found = true;
        }
        idx++;
      }
      if (!found) classNode.add(a);
    }

    if (packed) classNode = root;

    int attribCount = classNode.getChildCount();

    DefaultMutableTreeNode tempNode = new DefaultMutableTreeNode("methods",true);

    for (Enumeration e1 = ci.methods(); e1.hasMoreElements();){
      MethodInfo mi = (MethodInfo)e1.nextElement();
      DefaultMutableTreeNode a = new DefaultMutableTreeNode(mi, false);

      children = tempNode.children();
      idx = 0;
      found = false;
      while (children.hasMoreElements() && !found){
        DefaultMutableTreeNode cur = (DefaultMutableTreeNode)children.nextElement();
        String curName = (String)((MethodInfo)cur.getUserObject()).name;
        if (curName.compareTo(mi.name) > 0){
          tempNode.insert(a, idx);
          found = true;
        }
        idx++;
      }
      if (!found) tempNode.add(a);
    }

    // append method interface - adding to one node removes from other one!
    while (tempNode.getChildCount() > 0){
      classNode.add((DefaultMutableTreeNode)tempNode.getFirstChild());
    }
    return root;
  }

  // build the tree with package division
  public DefaultMutableTreeNode getFlatClassTree(DefaultMutableTreeNode root, boolean packed){
    Hashtable packages = new Hashtable();

    for (Enumeration e = classes(); e.hasMoreElements();){
      JavaClassInfo ci = (JavaClassInfo)((ClassNode)e.nextElement()).thisClass;

      String pname = ci.getPackage();

      if (pname.equals("")) pname = "<anonymous package>";

      DefaultMutableTreeNode p = (DefaultMutableTreeNode)packages.get(pname);
      if (p == null){   // package does not exist
        p = new DefaultMutableTreeNode(pname, true);
        packages.put(pname, p);
      }
//      Assert.Assert(p != null,"package not found."); /* disabled by PE */
      Utils.Assert.assert(p,"package not found."); /* inserted by PE */

      DefaultMutableTreeNode classNode = buildClassNode(ci, packed);

      // add class node sorted by class name
      int idx = 0;
      Enumeration children = p.children();
      boolean found = false;

      while (children.hasMoreElements() && !found){
        DefaultMutableTreeNode cur = (DefaultMutableTreeNode)children.nextElement();
        String curName = (String)((ClassInfo)cur.getUserObject()).name;
        if (curName.compareTo(ci.name) > 0){
          p.insert(classNode, idx);
          found = true;
        }
        idx++;
      }
      if (!found) p.add(classNode);
    }

    // sort packages
    for (Enumeration e = packages.elements(); e.hasMoreElements();){
      DefaultMutableTreeNode n = (DefaultMutableTreeNode)e.nextElement();

      int idx = 0;
      Enumeration children = root.children();
      boolean found = false;

      while (children.hasMoreElements() && !found){
        DefaultMutableTreeNode cur = (DefaultMutableTreeNode)children.nextElement();
        String curName = (String)cur.getUserObject();
        if (curName.compareTo((String)n.getUserObject()) > 0){
          root.insert(n, idx);
          found = true;
        }
        idx++;
      }
      if (!found) root.add(n);
    }

    return root;
  }

  public static Hierarchy newJavaHierarchy(){
    JavaClassInfo ji = new JavaClassInfo("java.lang.Object","");
    ji.valid = true;
    return new JavaHierarchy(ji);
  }
}
