// project: SID - Structured Interactive Documentation
// author : Dipl.Ing. Stefan Chiettini

package SID.Utils;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import Utils.*;

public final class tool{

	public static String osname;
	private static Hashtable stringPool;

	public static void key(String s){
		int c;
		try{c = System.in.read();}
		catch (IOException e){
			System.out.println("error waiting"); return;
		}
		System.out.println(s);
	}

	// Mouse clicks

	public static boolean leftDown(MouseEvent e){
		return ((e.getID() == MouseEvent.MOUSE_PRESSED) && (e.getModifiers() != BUTTON2_MASK) && (e.getModifiers() != MouseEvent.BUTTON3_MASK));
	}
	public static boolean leftUp(MouseEvent e){
		return ((e.getID() == MouseEvent.MOUSE_RELEASED) && (e.getModifiers() != BUTTON2_MASK) && (e.getModifiers() != MouseEvent.BUTTON3_MASK));
	}
	public static boolean leftDrag(MouseEvent e){
		return ((e.getID() == MouseEvent.MOUSE_DRAGGED) && (e.getModifiers() != MouseEvent.BUTTON2_MASK) && (e.getModifiers() != MouseEvent.BUTTON3_MASK));
	}

	public static boolean middleDown(MouseEvent e){
		return ((e.getID() == MouseEvent.MOUSE_PRESSED) && (e.getModifiers() == BUTTON2_MASK));
	}
	public static boolean middleDrag(MouseEvent e){
		return ((e.getID() == MouseEvent.MOUSE_DRAGGED) && (e.getModifiers() == MouseEvent.BUTTON2_MASK));
	}
	public static boolean middleUp(MouseEvent e){
		return ((e.getID() == MouseEvent.MOUSE_RELEASED) && (e.getModifiers() == BUTTON2_MASK));
	}

	public static boolean rightDown(MouseEvent e){
		return ((e.getID() == MouseEvent.MOUSE_PRESSED) && (e.getModifiers() == MouseEvent.BUTTON3_MASK));
	}
	public static boolean rightDrag(MouseEvent e){
		return ((e.getID() == MouseEvent.MOUSE_DRAGGED) && (e.getModifiers() == MouseEvent.BUTTON3_MASK));
	}
	public static boolean rightUp(MouseEvent e){
		return ((e.getID() == MouseEvent.MOUSE_RELEASED) && (e.getModifiers() == MouseEvent.BUTTON3_MASK));
	}

	// Font serialization, assume that font exists on target system
	public static void writeFont(Font font, DataOutputStream f) throws IOException{
		writeString(font.getName(), f);
		f.writeShort(font.getSize());
		f.writeShort(font.getStyle());
	}

	public static Font readFont(DataInputStream f) throws IOException{
		String s = readString(f);
		int size = f.readShort();
		int style = f.readShort();
		return FontPool.getFont(s, style, size);
	}

	// Color serialization
	public static void writeColor(Color col, DataOutputStream f) throws IOException{
		f.writeByte(ColorPool.getColIndex(col.getRed(), col.getGreen(), col.getBlue()));
	}

	public static Color readColor(DataInputStream f) throws IOException{
		return ColorPool.getColByIndex((int)f.readByte());
	}
	
	public static void writeBitSet(BitSet b, int max, DataOutputStream f) throws IOException{
		int size = b.size(), i;
		byte outBuffer = 0;
		if (max < size) size = max;
		f.writeShort((short)size);
		for (i = 0; i < size; i++){
			outBuffer <<= 1;
			if (b.get(i)){
				outBuffer |= (byte)1;
			}
			if ((i+1) % 8 == 0){
				f.writeByte(outBuffer);
				outBuffer = 0;
			}
		}
		if (i % 8 != 0){
			outBuffer <<= 8 - (i % 8);		// fill with zero at the right side
			f.writeByte(outBuffer);
		}
	}

	public static BitSet readBitSet(DataInputStream f) throws IOException{
		int size = f.readShort();

		byte inBuffer = 0;
		BitSet bs = new BitSet(size);
		int bytes = (int)(size / 8) + ((size % 8 == 0) ? 0:1);

		for (int i = 0; i < bytes; i++){
			inBuffer = f.readByte();
			
			for (int j = 0; j < 8; j++){
				if (((inBuffer >> (7-j)) & (byte)1) == (byte)1)	bs.set(i*8 + j);
			}
		}
		return bs;
	}

	public static void resetSerialization(){
		stringPool = new Hashtable(40);
	}

	public static void finalizeSerialization(){
		stringPool = null;
	}

	// serialization of class information
	public static void writeClassInfo(Object o, DataOutputStream f) throws IOException{
		String s = o.getClass().getName();
		writeString(s, f);
	}

	// load and create an empty object
	public static Object readClassInfo(DataInputStream f) throws IOException{
		Object o=null;
		String s = tool.readString(f);
		try{
			Class c = Class.forName(s);
			o = c.newInstance();
		}

		catch (ClassNotFoundException e1){
			System.out.println("class '"+s+"' not found.");
			System.out.println(e1.toString());
		}
		catch (InstantiationException e1){
			System.out.println("instantiation of class '"+s+"' failed.");
			System.out.println(e1.toString());
		}
		catch (IllegalAccessException e1){
			System.out.println("access to class '"+s+"' not accepted.");
			System.out.println(e1.toString());
		}
		finally{
			return o;
		}
	}

	public static void writeString(String s, DataOutputStream f) throws IOException{
		int hash = s.hashCode();
		f.writeInt(hash);
		Integer ihash = new Integer(hash);
		if (!stringPool.containsKey(ihash)){
			f.writeUTF(s);
			stringPool.put(ihash, s);
		}
	}
	
	public static String readString(DataInputStream f) throws IOException{
		String s;
		Integer hash = new Integer(f.readInt());
		Object o = stringPool.get(hash);
		if (o != null) s = (String)o;
		else{
			s = f.readUTF();
			stringPool.put(hash, s);
		}
		return s;
	}

/*	public static Dimension stringDim(Graphics g, String str, Font f){
		int H, W;
		Dimension d;

		FontMetrics fm = g.getFontMetrics(f);
		
		H = fm.getHeight();
		W = fm.stringWidth(str);
//
//		Versions before 1.1.5: workaround
//
		if (f.isItalic()){		// stringWidth only computes the advance width, not the width
			W += ((float)fm.getMaxAscent() * 0.4663);
			W += Math.round((float)fm.getMaxDescent() * 0.4663);
		}

		d = new Dimension(W, H);
		return d;
	}
*/

	public static Font copyFont(Font f){
		return new Font(f.getName(), f.getStyle(), f.getSize());
	}

	public static Dimension charsDim(char[] str, int len, Font f, Graphics g){
		FontMetrics fm = g.getFontMetrics(f);
		return new Dimension(fm.charsWidth(str, 0, len), fm.getHeight());
	}
	
	public static Dimension stringDim(Graphics g, String str, Font f){
		FontMetrics fm = g.getFontMetrics(f);
		return new Dimension(fm.stringWidth(str), fm.getHeight());
	}

	public static boolean isValidDir(String path){
		if (path == null) return false;
		File f = new File(path);
		return (f.exists() && f.isDirectory());
	}

/*	public static boolean isPrintable(KeyEvent evt){
		int mods = evt.getModifiers();
		return (((mods == 0) || (mods == KeyEvent.SHIFT_MASK)) && (evt.getID() == KeyEvent.KEY_TYPED));
	}
*/
	public static double getLineLength(int x1, int y1, int x2, int y2){
		return Math.sqrt((x1-x2) * (x1-x2) + (y1-y2)*(y1-y2));

	}

	public static void drawSelection(int x, int y, int w, int h, Graphics g){
		g.setXORMode(Color.white);
		g.fillRect(x-4, y-4, 4, 4); g.fillRect(x-4, y+h, 4, 4);
		g.fillRect(x+w, y-4, 4, 4); g.fillRect(x+w, y+h, 4, 4);
		g.setPaintMode();
		//System.out.println("draw Selection called");
	}

	// subtracts r1 from r2 and returns two resulting rectangles
	public static void subtract(Rectangle r1, Rectangle r2, Rectangle r3, Rectangle r4){
		int w, h;
		Rectangle i = r1.intersection(r2);
		
		if ((i.height == 0) && (i.width == 0)){
			r3.setBounds(0,0,0,0); r4.setBounds(0,0,0,0);
		}
		else if (i.height == r1.height){
			r3.setBounds(r1.x, r1.y, i.x-r1.x, i.height);
			r4.setBounds(i.x+i.width, r1.y, r1.width-i.width-r3.width, i.height);
		}
		else if (i.width == r1.width){
			r3.setBounds(r1.x, r1.y, i.width, i.y - r1.y);
			r4.setBounds(r1.x, i.y+i.height, i.width, r1.height-i.height-r3.height);
		}
		else if (i.x == r1.x){
			r3.setBounds(r1.x+i.width, r1.y, r1.width-i.width, r1.height);
			if (i.y == r1.y)
				r4.setBounds(r1.x, r1.y+i.height, i.width, r1.height-i.height);
			else
				r4.setBounds(r1.x, r1.y, i.width, r1.height-i.height);
		}
		else if (i.x + i.width == r1.x + r1.width){
			r3.setBounds(r1.x, r1.y, r1.width-i.width, r1.height);
			if (i.y == r1.y)
				r4.setBounds(r1.x+r3.width, r1.y+i.height, i.width, r1.height-i.height);
			else
				r4.setBounds(r1.x+r3.width, r1.y, i.width, r1.height-i.height);
		}
	}

	// return if a hotkey combination of CTRL and a given key c was pressed
	// c must be a virtual key code
	public static boolean Ctrl(KeyEvent e, int c){
		return ((e.getKeyCode() == c) && (e.getModifiers() == KeyEvent.CTRL_MASK));
	}

	// build a KEY_PRESS event with the target o and the key c combined with the CTRL-key
	// menu selections are mapped to hotkey-events, c must be avirtual key code
	public static KeyEvent buildCtrl(int c, Component o){
		return new KeyEvent(o, KeyEvent.KEY_PRESSED, 0, KeyEvent.CTRL_MASK, c);
	}

	// build a KEY_PRESS event with the target o and the key c
	// menu selections are mapped to hotkey-events, c must be a virtual key code
	public static KeyEvent buildKey(int c, Component o){
		return new KeyEvent(o, KeyEvent.KEY_PRESSED, 0, 0, c);
	}
	public static final int EOL = 100000;		// set caret at end of line, mild hack

	public static final int BUTTON2_MASK = 10;

	public static final String m_file = "File";
	public static final String m_file_new = "New";
	public static final String m_file_open = "Open";
	public static final String m_file_save = "Save";
	public static final String m_file_saveas = "Save As";
	public static final String m_file_exit = "Exit";

	public static final String m_edit = "Edit";
	public static final String m_edit_cut = "Cut";
	public static final String m_edit_copy = "Copy";
	public static final String m_edit_paste = "Paste";
	
	public static final String m_elements = "Elements";
	public static final String m_elem_class = "new Class";
	public static final String m_elem_method = "new Method";
	public static final String m_elem_textfield = "new TextField";
	public static final String m_elem_link = "new Link";
	public static final String m_elem_url = "new URL";
	public static final String m_elem_source = "new Source Link";
	public static final String m_elem_docu = "new Documentation Link";
	public static final String m_elem_version = "new Version Element";
	public static final String m_elem_picture = "insert Picture";
	public static final String m_elem_alignment = "alignment";
	public static final String m_elem_align_top = "top";
	public static final String m_elem_align_center = "center";
	public static final String m_elem_align_bottom = "bottom";
	public static final String m_elem_align_baseline = "baseline";
	public static final String m_elem_showobjects = "show invisible objects";
	public static final String m_elem_editurl = "edit URL";

	public static final String m_links = "Links";
	public static final String m_links_toggle = "toggle link style";
	public static final String m_links_del = "delete selected link";

	public static final String m_variants = "Variants";
	public static final String m_var_add = "add variant";
	public static final String m_var_del = "delete selected variant";
	public static final String m_var_next = "next variant";
	public static final String m_var_prev = "previous variant";

	public static final String m_options = "Options";
	public static final String m_options_browser = "Internet Browser";
	public static final String m_options_tabs = "Tabs";
	public static final String m_options_links = "Link style";
	public static final String m_options_save = "Save Options";

	public static final String fileSep = System.getProperty("file.separator");

	public static final String empty = "                                                                                                                                                                                        ";

	public static final Font defaultFont = new Font("TimesNewRoman", Font.PLAIN, 12);

}