/* ******************************************************************

   class    : OutputRedirector
   date     : Okt 14 1998
   compiler : javac (1.1.6)

******************************************************************* */

package SysExt;

/* ******************************************************************
   import statements
******************************************************************* */

import java.io.*;

/**
 * This class redirects the output stream or the
 * error stream to the standard output/error stream.
 *
 * @author puehringer edgar
 */
public class OutputRedirector implements Runnable {

  // instance variable declarations
  private InputStream is;
  private OutputStream os;
  /** True if auto flush is enabled. */
  protected boolean autoFlush;

  // constructors
  /** Create a new OutputRedirector object. */
  public OutputRedirector(Process proc, boolean err) {
    this(proc,err,false);
  } // end constructor OutputRedirector

  /** Create a new OutputRedirector object. */
  public OutputRedirector(Process proc, boolean err, boolean autoFlush) {
    is = err ? proc.getErrorStream() : proc.getInputStream();
    os = err ? System.err            : System.out;
    this.autoFlush = autoFlush;
    (new Thread(this)).start();
  } // end constructor OutputRedirector

  // methods
  //---- Runnable ---------------------------------------------------
  public void run() {
    int i;

    try {
      i = is.read();
      while (i != -1) {
        os.write(i);
        if (autoFlush) os.flush();
        i = is.read();
      } // end while
    } catch (IOException excep) {
      excep.printStackTrace();
    } finally {
      try { is.close(); } catch (IOException e) {}
    }
  } // end run

} // end OutputRedirector

/* ******************************************************************
   end of file
******************************************************************* */
