/* ******************************************************************

   class    : URLClassLoader
   date     : Apr 19 1999
   compiler : javac (1.2)

******************************************************************* */

package SysExt;

/* ******************************************************************
   import statements
******************************************************************* */

import java.io.*;
import java.net.*;

/**
 * A class loader that loads classes from an URL (codebase).
 * If the class can't be found, then the superclass (FileClassLoader)
 * is used to load the class.  This class can't handle jar files.
 *
 * @author Puehred
 */
public class URLClassLoader extends FileClassLoader {

  // instance variable declarations
  /** The codebase URL to load classes from. */
  private URL url;

  // constructors
  /**
   * Create a new URLClassLoader object.
   *
   * @param url The codebase URL to load classes from.
   */
  public URLClassLoader(URL url) {
    super();
    this.url = url;
  } // end constructor URLClassLoader

  /**
   * Create a new URLClassLoader object.
   *
   * @param url The codebase URL to load classes from.
   * @param path A additional path to add to the class loader.
   */
  public URLClassLoader(URL url, String path) {
    super(path);
    this.url = url;
  } // end constructor URLClassLoader

  // methods
  /** Finds the specified class. */
  public Class findClass(String name) throws ClassNotFoundException {
    InputStream is;
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    byte[] b                   = new byte[4096];
    URL u;
    String n = name.replace('.','/') + ".class";
    int nch;

    try {
      u = new URL(url,n);
    } catch (MalformedURLException excep) {
      throw new ClassNotFoundException("Malformed Class URL: '" + url +
                  n + "'.");
    }
    try {
      is = u.openStream();
    } catch (IOException excep) {
      // url doesn't exist!
      return super.findClass(name);
    }
    try {
      while ((nch = is.read(b,0,b.length)) != -1) {
        baos.write(b,0,nch);
      } // end while
    } catch (IOException excep) {
      throw new ClassNotFoundException("IOException on reading '" + u +
                  "':\n" + excep);
    } finally {
      try { if (is != null) is.close(); } catch (IOException e) {}
    }
    b = baos.toByteArray();
    return defineClass(name,b,0,b.length);
  } // end findClass

  /**
   * Finds the resource with the given name.
   */
  protected URL findResource(String name) {
    try {
      return new URL(url,name);
    } catch (MalformedURLException excep) {
      return null;
    }
  } // end findResource

} // end URLClassLoader

/* ******************************************************************
   end of file
******************************************************************* */
